module Substitute where



import Data.Map (Map)
import qualified Data.Map as Map

import Core
import Names



substituteTypeInType :: Map Name Type -> Type -> Type
substituteTypeInType subs (VarType name kind) =
	case Map.lookup name subs of
		Just sub -> sub
		Nothing -> VarType name kind
substituteTypeInType subs (ConType name kind) =
	ConType name kind
substituteTypeInType subs (ApplyType fun arg) =
	ApplyType (substituteTypeInType subs fun) (substituteTypeInType subs arg)
substituteTypeInType subs (FunctionType arg ret) =
	FunctionType (substituteTypeInFieldType subs arg) (substituteTypeInFlavorType subs ret)



substituteTypeInFlavorType :: Map Name Type -> FlavorType -> FlavorType
substituteTypeInFlavorType subs (FlavorType flavor ty) =
	FlavorType (substituteTypeInType subs flavor) (substituteTypeInType subs ty)



substituteTypeInFieldType :: Map Name Type -> FieldType -> FieldType
substituteTypeInFieldType subs (StrictField ty) =
    StrictField (substituteTypeInType subs ty)
substituteTypeInFieldType subs (LazyField flavor ty) =
    LazyField (substituteTypeInType subs flavor) (substituteTypeInType subs ty)



substituteTypeInExpr :: Map Name Type -> Expr -> Expr
substituteTypeInExpr subs (VarExpr (name, ft)) =
	VarExpr (name, substituteTypeInFlavorType subs ft)
substituteTypeInExpr subs (ConExpr (name, ft)) =
	ConExpr (name, substituteTypeInFlavorType subs ft)
substituteTypeInExpr subs (NumericLiteralExpr n f) =
	NumericLiteralExpr n (substituteTypeInType subs f)
substituteTypeInExpr subs (StringLiteralExpr s f) =
    StringLiteralExpr s (substituteTypeInType subs f)
substituteTypeInExpr subs (LambdaExpr name ty body) =
	LambdaExpr name (substituteTypeInFieldType subs ty) (substituteTypeInExpr subs body)
substituteTypeInExpr subs (ApplyExpr strictness fun arg) =
	ApplyExpr strictness (substituteTypeInExpr subs fun) (substituteTypeInExpr subs arg)
substituteTypeInExpr subs (CaseExpr subj branches) =
	CaseExpr (substituteTypeInExpr subs subj)
		[(substituteTypeInPattern subs pat, substituteTypeInExpr subs exp) | (pat, exp) <- branches]
substituteTypeInExpr subs (DoExpr stmts result) =
	DoExpr [case stmt of
		BindStmt name ty val ->
		    BindStmt name (substituteTypeInFieldType subs ty) (substituteTypeInExpr subs val)
		ActStmt val ->
		    ActStmt (substituteTypeInExpr subs val)
		| stmt <- stmts]
		(substituteTypeInExpr subs result)



substituteTypeInPattern :: Map Name Type -> Pattern -> Pattern
substituteTypeInPattern subs (VarPattern (name, ty)) =
	VarPattern (name, substituteTypeInFieldType subs ty)
substituteTypeInPattern subs (ConApplyPattern (con, ty) pats) =
	ConApplyPattern (con, substituteTypeInType subs ty)
	    [(strictness, substituteTypeInPattern subs pat) | (strictness, pat) <- pats]
substituteTypeInPattern subs WildcardPattern =
	WildcardPattern
